@@ -1,22 +1,22 @@
 use crate::{
-    cache::ProtofetchCache,
+    cache::ProtofetchGitCache,
     fetch,
     model::{
         protodep::ProtodepDescriptor,
         protofetch::{Descriptor, LockFile},
     },
+    proto,
 };
 use std::{
     env,
     error::Error,
-    fs,
     path::{Path, PathBuf},
 };
 
 /// Handler to fetch command
 pub fn do_fetch(
     force_lock: bool,
-    cache: &ProtofetchCache,
+    cache: &ProtofetchGitCache,
     conf_path: &Path,
     lockfile_path: &Path,
     dependencies_out_dir: &Path,
@@ -34,8 +34,9 @@ pub fn do_fetch(
         .as_ref()
         .map(Path::new)
         .unwrap_or(proto_output_directory);
-    fetch::fetch(cache, &lockfile, &cache_src_dir, proto_out_dir)?;
-
+    fetch::fetch_sources(cache, &lockfile, &cache_src_dir)?;
+    //Copy proto_out files to actual target
+    proto::copy_proto_files(proto_out_dir, &cache_src_dir, &lockfile)?;
     Ok(())
 }
 
@@ -43,7 +44,7 @@ pub fn do_fetch(
 /// Loads dependency descriptor from protofetch toml or protodep toml
 /// Generates a lock file based on the protofetch.toml
 pub fn do_lock(
-    cache: &ProtofetchCache,
+    cache: &ProtofetchGitCache,
     conf_path: &Path,
     lockfile_path: &Path,
 ) -> Result<LockFile, Box<dyn Error>> {
@@ -59,8 +60,8 @@ pub fn do_lock(
     let lockfile = fetch::lock(&module_descriptor, cache)?;
 
     log::debug!("Generated lockfile: {:?}", lockfile);
-
-    std::fs::write(lockfile_path, toml::to_string_pretty(&lockfile)?)?;
+    let value_toml = toml::Value::try_from(&lockfile)?;
+    std::fs::write(lockfile_path, toml::to_string_pretty(&value_toml)?)?;
 
     log::info!("Wrote lockfile to {}", lockfile_path.to_string_lossy());
 
@@ -118,11 +119,24 @@ pub fn do_clean(lockfile_path: &Path, proto_output_directory: &Path) -> Result<(
             .map(Path::new)
             .unwrap_or(proto_output_directory);
         info!(
-            "Cleaning protofetch proto source files folder {}.",
+            "Cleaning protofetch proto_out source files folder {}.",
             &proto_out_dir.to_string_lossy()
         );
-        fs::remove_dir_all(proto_out_dir)?;
-        fs::remove_file(lockfile_path).expect("Lockfile could not be removed");
+        std::fs::remove_dir_all(proto_out_dir)?;
+        std::fs::remove_file(lockfile_path).expect("Lockfile could not be removed");
+        Ok(())
+    } else {
+        Ok(())
+    }
+}
+
+pub fn do_clear_cache(cache: &ProtofetchGitCache,) -> Result<(), Box<dyn Error>> {
+    if cache.location.exists() {
+        info!(
+            "Clearing protofetch repository cache {}.",
+            &cache.location.to_string_lossy()
+        );
+        std::fs::remove_dir_all(&cache.location)?;
         Ok(())
     } else {
         Ok(())
