@@ -10,7 +10,7 @@ use crate::{
     proto_repository::ProtoRepository,
 };
 
-use crate::model::protofetch::Descriptor;
+use crate::model::protofetch::{DependencyName, Descriptor, Rules};
 use thiserror::Error;
 
 #[derive(Error, Debug)]
@@ -39,47 +39,85 @@ pub fn lock<Cache: RepositoryCache>(
 ) -> Result<LockFile, FetchError> {
     fn go<Cache: RepositoryCache>(
         cache: &Cache,
-        dep_map: &mut HashMap<Coordinate, Vec<Revision>>,
-        repo_map: &mut HashMap<Coordinate, (String, ProtoRepository)>,
+        dep_map: &mut HashMap<DependencyName, Vec<Revision>>,
+        repo_map: &mut HashMap<
+            DependencyName,
+            (Rules, Coordinate, ProtoRepository, Vec<DependencyName>),
+        >,
         dependencies: &[Dependency],
+        parent: Option<&DependencyName>,
     ) -> Result<(), FetchError> {
         for dependency in dependencies {
             log::info!("Resolving {:?}", dependency.coordinate);
 
             dep_map
-                .entry(dependency.coordinate.clone())
+                .entry(dependency.name.clone())
                 .and_modify(|vec| vec.push(dependency.revision.clone()))
                 .or_insert_with(|| vec![dependency.revision.clone()]);
 
             let repo = cache.clone_or_update(&dependency.coordinate)?;
             let descriptor = repo.extract_descriptor(&dependency.name, &dependency.revision)?;
 
-            repo_map
-                .entry(dependency.coordinate.clone())
-                .or_insert((dependency.name.clone(), repo));
+            repo_map.entry(dependency.name.clone()).or_insert((
+                dependency.rules.clone(),
+                dependency.coordinate.clone(),
+                repo,
+                vec![],
+            ));
 
-            go(cache, dep_map, repo_map, &descriptor.dependencies)?;
+            if let Some(p) = parent {
+                repo_map
+                    .entry(p.clone())
+                    .and_modify(|(_r, _c, _p, deps)| deps.push(dependency.name.clone()));
+            }
+            go(
+                cache,
+                dep_map,
+                repo_map,
+                &descriptor.dependencies,
+                Some(&dependency.name),
+            )?;
         }
 
         Ok(())
     }
 
-    let mut dep_map: HashMap<Coordinate, Vec<Revision>> = HashMap::new();
-    let mut repo_map: HashMap<Coordinate, (String, ProtoRepository)> = HashMap::new();
+    let mut dep_map: HashMap<DependencyName, Vec<Revision>> = HashMap::new();
+    let mut repo_map: HashMap<
+        DependencyName,
+        (Rules, Coordinate, ProtoRepository, Vec<DependencyName>),
+    > = HashMap::new();
 
-    go(cache, &mut dep_map, &mut repo_map, &descriptor.dependencies)?;
+    go(
+        cache,
+        &mut dep_map,
+        &mut repo_map,
+        &descriptor.dependencies,
+        None,
+    )?;
 
     let no_conflicts = resolve_conflicts(dep_map);
-    let with_repos: HashMap<Coordinate, (String, ProtoRepository, Revision)> = no_conflicts
+    let with_revision: HashMap<
+        DependencyName,
+        (
+            Rules,
+            Coordinate,
+            ProtoRepository,
+            Revision,
+            Vec<DependencyName>,
+        ),
+    > = no_conflicts
         .into_iter()
         .filter_map(|(coordinate, revision)| {
             repo_map
                 .remove(&coordinate)
-                .map(|(dep_name, repo)| (coordinate, (dep_name, repo, revision)))
+                .map(|(rules, dep_name, repo, deps)| {
+                    (coordinate, (rules, dep_name, repo, revision, deps))
+                })
         })
         .collect();
 
-    let locked_dependencies = locked_dependencies(&with_repos)?;
+    let locked_dependencies = locked_dependencies(&with_revision)?;
 
     Ok(LockFile::new(
         descriptor.name.clone(),
@@ -88,11 +126,10 @@ pub fn lock<Cache: RepositoryCache>(
     ))
 }
 
-pub fn fetch<Cache: RepositoryCache>(
+pub fn fetch_sources<Cache: RepositoryCache>(
     cache: &Cache,
     lockfile: &LockFile,
     cache_src_dir: &Path,
-    proto_out_dir: &Path,
 ) -> Result<(), FetchError> {
     info!("Fetching dependencies source files...");
 
@@ -104,11 +141,11 @@ pub fn fetch<Cache: RepositoryCache>(
         for dep in &lockfile.dependencies {
             //If the dependency is already in the cache, we don't need to fetch it again
             if cache_src_dir
-                .join(&dep.name)
+                .join(&dep.name.value)
                 .join(PathBuf::from(&dep.commit_hash))
                 .exists()
             {
-                debug!("Skipping fetching {}. Already in cache", dep.name);
+                debug!("Skipping fetching {:?}. Already in cache", dep.name);
                 continue;
             }
             let repo = cache.clone_or_update(&dep.coordinate)?;
@@ -124,8 +161,6 @@ pub fn fetch<Cache: RepositoryCache>(
                 Please delete the cache and run protofetch fetch again.")
             }
         }
-        //Copy proto files to actual target
-        copy_proto_files(proto_out_dir, cache_src_dir, lockfile)?;
         Ok(())
     } else {
         Err(FetchError::BadOutputDir(
@@ -133,74 +168,12 @@ pub fn fetch<Cache: RepositoryCache>(
         ))
     }
 }
-/// proto_out_dir: Base path to the directory where the proto files are to be copied to
-/// cache_src_dir: Base path to the directory where the dependencies sources are cached
-/// lockfile: The lockfile that contains the dependencies to be copied
-pub fn copy_proto_files(
-    proto_out_dir: &Path,
-    cache_src_dir: &Path,
-    lockfile: &LockFile,
-) -> Result<(), FetchError> {
-    info!("Copying proto files...");
-    if !proto_out_dir.exists() {
-        std::fs::create_dir_all(proto_out_dir)?;
-    }
-
-    for dep in &lockfile.dependencies {
-        debug!("Copying proto files for {}", dep.name.as_str());
-        let dep_dir = cache_src_dir.join(&dep.name).join(&dep.commit_hash);
-        for file in dep_dir.read_dir()? {
-            let path = file?.path();
-            let proto_files = find_proto_files(path.as_path())?;
-            for proto_file_source in proto_files {
-                trace!(
-                    "Copying proto file {}",
-                    &proto_file_source.to_string_lossy()
-                );
-                let proto_src = proto_file_source.strip_prefix(&dep_dir).map_err(|_err| {
-                    FetchError::BadOutputDir(format!(
-                        "Could not create proto source path in {}. Wrong base dir {}",
-                        proto_file_source.to_string_lossy(),
-                        dep_dir.to_string_lossy()
-                    ))
-                })?;
-                let proto_out_dist = proto_out_dir.join(&proto_src);
-                let prefix = proto_out_dist.parent().ok_or_else(|| {
-                    FetchError::BadFilePath(format!(
-                        "Bad parent dest file for {}",
-                        &proto_out_dist.to_string_lossy()
-                    ))
-                })?;
-                std::fs::create_dir_all(prefix)?;
-                std::fs::copy(proto_file_source.as_path(), proto_out_dist.as_path())?;
-            }
-        }
-    }
-    Ok(())
-}
-
-fn find_proto_files(dir: &Path) -> Result<Vec<PathBuf>, FetchError> {
-    let mut files: Vec<PathBuf> = Vec::new();
-    if dir.is_dir() {
-        for entry in std::fs::read_dir(dir)? {
-            let entry = entry?;
-            let path = entry.path();
-            if path.is_dir() {
-                let rec_call = find_proto_files(&path)?;
-                files.append(&mut rec_call.clone());
-            } else if let Some(extension) = path.extension() {
-                if extension == "proto" {
-                    files.push(path);
-                }
-            }
-        }
-    }
-    Ok(files)
-}
 
 //TODO: Make sure we get the last version. Getting the biggest string is extremely error prone.
 //      Use semver
-fn resolve_conflicts(dep_map: HashMap<Coordinate, Vec<Revision>>) -> HashMap<Coordinate, Revision> {
+fn resolve_conflicts(
+    dep_map: HashMap<DependencyName, Vec<Revision>>,
+) -> HashMap<DependencyName, Revision> {
     dep_map
         .into_iter()
         .filter_map(|(k, mut v)| {
@@ -211,7 +184,7 @@ fn resolve_conflicts(dep_map: HashMap<Coordinate, Vec<Revision>>) -> HashMap<Coo
                 1 => Some((k, v.remove(0))),
                 _ => {
                     log::warn!(
-                        "discarded {} dependencies while resolving conflicts for {}",
+                        "discarded {} dependencies while resolving conflicts for {:?}",
                         len - 1,
                         k
                     );
@@ -222,11 +195,20 @@ fn resolve_conflicts(dep_map: HashMap<Coordinate, Vec<Revision>>) -> HashMap<Coo
         .collect()
 }
 
-pub fn locked_dependencies(
-    dep_map: &HashMap<Coordinate, (String, ProtoRepository, Revision)>,
+fn locked_dependencies(
+    dep_map: &HashMap<
+        DependencyName,
+        (
+            Rules,
+            Coordinate,
+            ProtoRepository,
+            Revision,
+            Vec<DependencyName>,
+        ),
+    >,
 ) -> Result<Vec<LockedDependency>, FetchError> {
     let mut locked_deps: Vec<LockedDependency> = Vec::new();
-    for (coordinate, (name, repository, revision)) in dep_map {
+    for (name, (rules, coordinate, repository, revision, deps)) in dep_map {
         log::info!("Locking {:?} at {:?}", coordinate, revision);
 
         let commit_hash = repository.resolve_commit_hash(revision, coordinate.branch.clone())?;
@@ -234,6 +216,8 @@ pub fn locked_dependencies(
             name: name.clone(),
             commit_hash,
             coordinate: coordinate.clone(),
+            dependencies: deps.clone(),
+            rules: rules.clone(),
         };
 
         locked_deps.push(locked_dep);
@@ -244,16 +228,10 @@ pub fn locked_dependencies(
 
 #[test]
 fn remove_duplicates() {
-    let mut input: HashMap<Coordinate, Vec<Revision>> = HashMap::new();
-    let mut result: HashMap<Coordinate, Revision> = HashMap::new();
-    let coordinate = Coordinate::new(
-        "github.com".to_string(),
-        "test".to_string(),
-        "test".to_string(),
-        crate::model::protofetch::Protocol::Https,
-        None,
-    );
-    input.insert(coordinate.clone(), vec![
+    let mut input: HashMap<DependencyName, Vec<Revision>> = HashMap::new();
+    let mut result: HashMap<DependencyName, Revision> = HashMap::new();
+    let name = DependencyName::new("foo".to_string());
+    input.insert(name.clone(), vec![
         Revision::Arbitrary {
             revision: "1.0.0".to_string(),
         },
@@ -264,7 +242,7 @@ fn remove_duplicates() {
             revision: "2.0.0".to_string(),
         },
     ]);
-    result.insert(coordinate, Revision::Arbitrary {
+    result.insert(name, Revision::Arbitrary {
         revision: "3.0.0".to_string(),
     });
     assert_eq!(resolve_conflicts(input), result)
